#import "FlutterTimPlugin.h"
#import "MJExtension.h"
#import "ImSDK.h"
#import "DimModel.h"
#import "YYModel.h"


@interface FlutterTimPlugin() <TIMConnListener, TIMUserStatusListener, TIMRefreshListener, TIMMessageListener>
@end

@implementation FlutterTimPlugin{
    FlutterEventSink _eventMsgSink;
    FlutterEventSink _eventConversionSink;
}
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"tim_method"
            binaryMessenger:[registrar messenger]];
  FlutterTimPlugin* instance = [[FlutterTimPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
    
    FlutterEventChannel *eventMsgChannel = [FlutterEventChannel eventChannelWithName:@"tim_event_msg" binaryMessenger:[registrar messenger]];
    
//    [eventMsgChannel setStreamHandler:(NSObject<FlutterStreamHandler> * _Nullable)];
    
    FlutterEventChannel *eventConversationChannel = [FlutterEventChannel eventChannelWithName:@"tim_event_msg" binaryMessenger:[registrar messenger]];
//    [eventConversationChannel setStreamHandler:(NSObject<FlutterStreamHandler> * _Nullable)];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  }else if([@"imLogin" isEqualToString:call.method]){//登录初始化开始

        NSString *appId = (NSString *)call.arguments[@"sdkAppId"];
      
        NSString *identifier = (NSString *)(call.arguments[@"identifier"]);
        NSString *userSig = (NSString *)(call.arguments[@"userSig"]);
        
        //初始化 SDK 基本配置
        TIMSdkConfig *config = [TIMSdkConfig new];
        config.sdkAppId = [appId intValue];
        config.connListener = self;
        
        //初始化 SDK
        [[TIMManager sharedInstance] initSdk:config];
        //将用户配置与通讯管理器进行绑定
        TIMUserConfig *userConfig = [TIMUserConfig new];
        userConfig.userStatusListener = self;
        userConfig.refreshListener = self;
        [[TIMManager sharedInstance] setUserConfig:userConfig];
        [[TIMManager sharedInstance] addMessageListener:self];
        TIMLoginParam *login_param = [[TIMLoginParam alloc ]init];
        // identifier 为用户名，userSig 为用户登录凭证
        // appidAt3rd 在私有帐号情况下，填写与 sdkAppId 一样
        login_param.identifier = identifier;
        login_param.userSig = userSig;
        //      login_param.appidAt3rd = appId;
        [[TIMManager sharedInstance] login: login_param succ:^(){
            result(@"Login Succ");
        } fail:^(int code, NSString * err) {
            NSLog(@"Login Failed: %d->%@", code, err);
            result([NSString stringWithFormat:@"Login Failed: %d->%@", code, err]);
        }];
    }
    //登录初始化结束
    //退出
    else if ([ @"imLogout" isEqualToString:call.method] ){
        [[TIMManager sharedInstance] logout:^{
            result(@"Logout Succ");
        } fail:^(int code, NSString *msg) {
            result([NSString stringWithFormat:@"Login Failed: %d->%@", code, msg]);
        }];
    }
    //获取会话列表
    else if([@"getConversationList" isEqualToString:call.method]){
        
        NSArray *conversationList = [[TIMManager sharedInstance] getConversationList];
        if (conversationList!=nil && conversationList.count>0) {
            NSMutableArray *dictArray = [[NSMutableArray alloc]init];
            for (TIMConversation *conversation in conversationList) {
                DimConversation *dimConversation = [DimConversation initWithTIMConversation:conversation ];
                [dictArray addObject:dimConversation];
            }
            NSString *jsonString = [dictArray yy_modelToJSONString];
            result(jsonString);
        }else{
            result(@"[]");
        }
    }else if([@"deleteConversation" isEqualToString:call.method]){
        NSString *identifier = call.arguments[@"identifier"];
        [[TIMManager sharedInstance] deleteConversation:TIM_C2C receiver:identifier];
        result(@"delConversation success");
    }else if([@"getMessages" isEqualToString:call.method]){
        NSString *identifier = call.arguments[@"identifier"];
//        int count = [call.arguments[@"count"] intValue];
        int ctype = [call.arguments[@"ctype"] intValue];
        //TIMMessage *lastMsg = call.arguments[@"lastMsg"];
        TIMConversation *con = [[TIMManager sharedInstance] getConversation: ctype==2 ? TIM_GROUP:TIM_C2C receiver:identifier];
        [con getMessage:20 last:NULL succ:^(NSArray *msgs) {
            if(msgs != nil && msgs.count > 0){
                NSMutableArray *dictArray = [[NSMutableArray alloc]init];
                for (TIMMessage *message in msgs) {
                    DimMessage *dimMessage = [DimMessage initWithTIMMessage:message];
                    [dictArray addObject:dimMessage];
                }
                NSString *jsonString = [dictArray yy_modelToJSONString];
                result(jsonString);
            }else{
                result(@"[]");
            }
        } fail:^(int code, NSString *msg) {
            result([NSString stringWithFormat:@"get message failed. code: %d msg: %@", code, msg]);
        }];
    }else if([@"sendTextMessages" isEqualToString:call.method]){
        NSString *identifier = call.arguments[@"identifier"];
        NSString *content = call.arguments[@"content"];
        TIMMessage *msg = [TIMMessage new];
        
        //添加文本内容
        TIMTextElem *elem = [TIMTextElem new];
        elem.text = content;
        
        //将elem添加到消息
        if([msg addElem:elem] != 0){
            NSLog(@"addElement failed");
            return;
        }
        TIMConversation *conversation = [[TIMManager sharedInstance] getConversation:TIM_C2C receiver:identifier];
        //发送消息
        [conversation sendMessage:msg succ:^{
            result(@"send message ok");
        } fail:^(int code, NSString *msg) {
            result([NSString stringWithFormat:@"send message failed. code: %d desc:%@", code, msg]);
        }];
    }else if([@"sendImageMessages" isEqualToString:call.method]){
        NSString *identifier = call.arguments[@"identifier"];
        NSString *iamgePath = call.arguments[@"image_path"];
        //构造一条消息
        TIMMessage *msg = [TIMMessage new];
        
        //添加图片
        TIMImageElem *elem = [TIMImageElem new];
        elem.path = iamgePath;
        if([msg addElem:elem] != 0){
            NSLog(@"addElement failed");
        }
        
        TIMConversation *conversation = [[TIMManager sharedInstance] getConversation:TIM_C2C receiver:identifier];
        [conversation sendMessage:msg succ:^{
            result(@"SendMsg ok");
        } fail:^(int code, NSString *msg) {
            result([NSString stringWithFormat:@"send message failed. code: %d desc:%@", code, msg]);
        }];
        
    }
    else {
    result(FlutterMethodNotImplemented);
  }
}


#pragma mark - FlutterStreamHandler
- (FlutterError*)onListenWithArguments:(id)arguments
                             eventSink:(FlutterEventSink)eventSink {
    _eventMsgSink = eventSink;
   
    return nil;
}

- (FlutterError* _Nullable)onCancelWithArguments:(id _Nullable)arguments{
    return nil;
}

#pragma mark - TIMMessageListener
/**
 *  新消息回调通知
 *
 *  @param msgs 新消息列表，TIMMessage 类型数组
 */
- (void)onNewMessage:(NSArray*)msgs{
    if(msgs != nil && msgs.count > 0){
        NSMutableArray *dictArray = [[NSMutableArray alloc]init];
        for (TIMMessage *message in msgs) {
            DimMessage *dimMessage = [DimMessage initWithTIMMessage:message];
            [dictArray addObject:dimMessage];
        }
        NSString *jsonString = [dictArray yy_modelToJSONString];
        _eventMsgSink(jsonString);
    }
}

#pragma mark - TIMRefreshListener
/**
 *  会话列表变动
 */
- (void)onRefresh{
//    self.eventSink(@"[]");
}
-(void) onRefreshConversations:(NSArray *)conversations{
    
}
@end
